;;; -*- Mode: Lisp -*-
;;;
;;; ilogic/convert.l --- Ilust Logic for xyzzy

;;; Copyright (C) 2001 OHKUBO Hiroshi

;;; Author: OHKUBO Hiroshi
;;; Time-stamp: <2001/10/06 15:28:04 +0900>

;;; Code

(provide "ilogic/convert")

(in-package "ilogic")

;(eval-when (:compile-toplevel :load-toplevel :execute)
;  (require "ilogic/data"))

(defun convert-directory (dir)
  (interactive "DOriginal Data Directory: ")
  (let (file-name-list
	file)
    (when (and (file-exist-p dir) (file-directory-p dir))
      (setq file-name-list (directory dir :file-only t))
      (delete "./" file-name-list :test #'string=)
      (setq file-name-list (sort file-name-list #'string<))
      
      (dolist (file-name file-name-list)
	(setq file (merge-pathnames file-name dir))
	(cond
	 ((convert-puzzlepalace-file file)
	  (message "pYpX`F ~A" file-name))
	 (t
	  (message "`sF ~A" file-name))))
      )))

(defun convert-puzzlepalace-file (file)
  (save-excursion
    (let ((input-file (map-backslash-to-slash file))
	  class-directory
	  output-file
	  output-file-name
	  number-str
	  name-str date-str level-str size-str timestamp-str
	  question-str solution-str author-str comment-str
	  ilogicdata
	  ilogicdata-class ilogicdata-title ilogicdata-problem ilogicdata-level
	  ilogicdata-answer iloigcdata-comment ilogicdata-height ilogicdata-width
	  ilogicdata-input ilogicdata-date
	  ilogicdata-author-name ilogicdata-author-email ilogicdata-author-url
	  )
      (unless (file-exist-p input-file)
	(return-from convert-puzzlepalace-file nil))
      (setq output-file-name
	    (change-suffix-filename (file-namestring input-file) "dat"))
      (set-temp-buffer)
      (insert-file-contents input-file)
      (goto-char (point-min))
      (unless (scan-buffer "<problem>")
	(return-from convert-puzzlepalace-file nil))
      (flet ((get-string (tag-str)
	       (let (from to)
		 (if (scan-buffer (concat "<" tag-str ">") :tail t)
		     (setq from (point)))
		 (if (scan-buffer (concat "</" tag-str ">") :tail nil)
		     (setq to (point)))
		 (unless (and from to)
		   (return-from convert-puzzlepalace-file nil))
		 (buffer-substring from to))))
	(setq name-str (get-string "name"))
	(setq date-str (get-string "date"))
	(setq level-str (get-string "level"))
	(setq size-str (get-string "size"))
	(setq timestamp-str (get-string "timestamp"))
	(setq question-str (get-string "question"))
	(setq solution-str (get-string "solution"))
	(setq author-str (get-string "author"))
	(setq comment-str (get-string "comment"))
	)
      (if (string-match "[0-9]+" (file-namestring input-file))
	  (setq number-str (match-string 0)))

      (let ((in (make-string-input-stream size-str)))
	(setq ilogicdata-width (read in nil))
	(setq ilogicdata-height (read in nil)))
      (let* ((work-list (split-string question-str #\@))
	     (left-input-str (car work-list))
	     (upper-input-str (cadr work-list))
	     left-input upper-input line-input)
	(dolist (line-input-str (split-string left-input-str #\x0A))
	  (setq line-input nil)
	  (let ((in (make-string-input-stream line-input-str))
		num)
	    (while (setq num (read in nil))
	      (push num line-input)))
	  (push (reverse line-input) left-input))
	(dolist (line-input-str (split-string upper-input-str #\x0A))
	  (setq line-input nil)
	  (let ((in (make-string-input-stream line-input-str))
		num)
	    (while (setq num (read in nil))
	      (push num line-input)))
	  (push (reverse line-input) upper-input))
	(setq ilogicdata-input (list (reverse left-input) (reverse upper-input))))

      (setq ilogicdata-class "convert/pYpX")
      (if (string= "" name-str)
	  (setq name-str "No Title"))
      (if number-str
	  (setq ilogicdata-title (concat name-str " " number-str))
	(setq ilogicdata-title name-str))
      (setq ilogicdata-problem "")
      (setq ilogicdata-level level-str)
      (setq ilogicdata-answer "")
      ;    (setq ilogicdata-comment comment-str)
      (setq ilogicdata-comment "")
      ;; date-str  universal-time  decode ̂蔲
      (setq ilogicdata-date date-str)
;      (setq ilogicdata-date timestamp-str)
      (if (string= "" author-str)
	  (progn
	    (setq ilogicdata-author-name "pYpX  copyright (C) by TIME INTERMEDIA")
	    (setq ilogicdata-author-email "palace@puzzle.gr.jp")
	    (setq ilogicdata-author-url "http://www.puzzle.gr.jp/"))
	(progn
	  (setq ilogicdata-author-name author-str)
	  (setq ilogicdata-author-email "")
	  (setq ilogicdata-author-url "")))


      (setq ilogicdata
	    (make-ilogicdata :class   ilogicdata-class
			     :title   ilogicdata-title
			     :problem ilogicdata-problem
			     :level   ilogicdata-level
			     :answer  ilogicdata-answer
			     :comment ilogicdata-comment
			     :input   ilogicdata-input
			     :width   ilogicdata-width
			     :height  ilogicdata-height
			     :date    ilogicdata-date
			     :author  (make-author
				       :name  ilogicdata-author-name
				       :email ilogicdata-author-email
				       :url   ilogicdata-author-url
				       )
			     :autosolve nil
			     ))

      (setq class-directory
	    (merge-pathnames (delete-unsuitable-filename-char ilogicdata-class)
			     *data-directory*))
      (unless (file-exist-p class-directory)
	(create-directory class-directory))
      (setq output-file (merge-pathnames output-file-name class-directory))
      (when (file-exist-p output-file)
	(setq output-file
	      (merge-pathnames (get-new-file-name class-directory)
			       class-directory)))
      (with-open-file (s output-file
			 :direction :output
			 :if-does-not-exist :create)
	(format s "~S\n" ilogicdata))
      t)))

(defun change-suffix-filename (filename new)
  (let ((str filename))
    (when (string-match "^\\(.+\\)\\." str)
      (setq str (match-string 1)))
    (concat str "." new)))
