;;; -*- Mode: Lisp -*-
;;;
;;; ilogic/draw.l --- Ilust Logic for xyzzy

;;; Copyright (C) 2001 OHKUBO Hiroshi

;;; Author: OHKUBO Hiroshi
;;; Time-stamp: <2001/10/10 09:52:02 +0900>

;;; Code

(provide "ilogic/draw")

(in-package "ilogic")

;(eval-when (:compile-toplevel :load-toplevel :execute)
;  (require "ilogic/data"))

(defvar *draw-auto-update* nil)

(defvar *buffer-draw* "I-Logic: Draw")

(defvar *draw-mode-hook* nil)

(defvar *draw-map* nil)
(setq *draw-map* (make-sparse-keymap))
(define-key *draw-map* #\RET 'draw-overwrite-on)
(define-key *draw-map* #\SPC 'draw-overwrite-question)
(define-key *draw-map* #\TAB 'draw-overwrite-toggle)
(define-key *draw-map* #\Left 'draw-left)
(define-key *draw-map* #\Right 'draw-right)
(define-key *draw-map* #\Up 'draw-up)
(define-key *draw-map* #\Down 'draw-down)
(define-key *draw-map* #\C-Left 'draw-paint-left)
(define-key *draw-map* #\C-Right 'draw-paint-right)
(define-key *draw-map* #\C-Up 'draw-paint-up)
(define-key *draw-map* #\C-Down 'draw-paint-down)
(define-key *draw-map* #\C-S-Left 'draw-erase-left)
(define-key *draw-map* #\C-S-Right 'draw-erase-right)
(define-key *draw-map* #\C-S-Up 'draw-erase-up)
(define-key *draw-map* #\C-S-Down 'draw-erase-down)
(define-key *draw-map* #\u 'draw-print-matrix-update)
(define-key *draw-map* #\t 'draw-auto-update-toggle)
(define-key *draw-map* '(#\C-x #\C-s) 'draw-infoinput-save-data)
(define-key *draw-map* '(#\C-c #\q) 'draw-delete)

(defun set-draw-buffer ()
  (set-buffer (get-buffer-create *buffer-draw*))
  (draw-mode))

(defun draw-mode ()
  (setq buffer-mode 'ilogic-draw)
  (setq mode-name *mode-name*)
  (make-local-variable 'input-data)
  (make-local-variable 'title-bar-format)
  (setq title-bar-format (format nil "~A - %b" *version-name*))
  (setq buffer-read-only t)
  (setq need-not-save t)
  (setq kept-undo-information t)
  (setq auto-save nil)
  (toggle-ime nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-line-number* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-newline* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-eof* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-cursor-line* nil)
  (set-buffer-fold-width nil)
  (use-keymap *draw-map*)
  (run-hooks '*draw-mode-hook*))

(defun draw-delete ()
  (interactive)
  (delete-buffer *buffer-draw*)
  (if (find-buffer *buffer-infoinput*)
      (infoinput-delete))
  (menu-show))

(defun draw-print-matrix-base (height width)
  (let (matrix-simple-info)
    (setq input-data (make-sparse-input height width))
    (setq matrix-simple-info (make-matrix-simple-info input-data))
    (draw-print-matrix input-data matrix-simple-info)
    (goto-char (point-min))
    (draw-goto-inside-matrix)
    ))

(defun draw-print-matrix (input &optional matrix-simple-info matrix-only)
  (let ((buffer-read-only nil)
	(indication (make-indication input)))
    (declare (special buffer-read-only))
    (erase-buffer (selected-buffer))
    (print-matrix-base indication matrix-simple-info matrix-only)))

(defun draw-inside-matrix-p ()
  (if (looking-at "[~]") t nil))

(defun draw-goto-inside-matrix ()
  (interactive)
  (unless (scan-buffer "[~]" :regexp t)
    (goto-char (point-min))
    (scan-buffer "[~]" :regexp t)))

(defun draw-overwrite-on ()
  (interactive)
  (let ((buffer-read-only nil))
    (declare (special buffer-read-only))
    (if (draw-inside-matrix-p)
	(unless (looking-for "")
	  (overwrite-char #\)
	  (backward-char)
	  (if *draw-auto-update* (draw-print-matrix-update)))
      (draw-goto-inside-matrix))))

(defun draw-overwrite-off ()
  (interactive)
  (let ((buffer-read-only nil))
    (declare (special buffer-read-only))
    (if (draw-inside-matrix-p)
	(unless (looking-for "~")
	  (overwrite-char #\~)
	  (backward-char)
	  (if *draw-auto-update* (draw-print-matrix-update)))
      (draw-goto-inside-matrix))))

(defun draw-overwrite-question ()
  (interactive)
  (let ((buffer-read-only nil))
    (declare (special buffer-read-only))
    (if (draw-inside-matrix-p)
	(unless (looking-for "")
	  (overwrite-char #\)
	  (backward-char)
	  (if *draw-auto-update* (draw-print-matrix-update)))
      (draw-goto-inside-matrix))))

(defun draw-overwrite-toggle ()
  (interactive)
  (if (draw-inside-matrix-p)
      (progn
	(case (schar (buffer-substring (point) (1+ (point))) 0)
	  (#\ (draw-overwrite-question))
	  (#\~ (draw-overwrite-on))
	  (#\ (draw-overwrite-on)))
;	(case (schar (buffer-substring (point) (1+ (point))) 0)
;	  (#\ (draw-overwrite-off))
;	  (#\~ (draw-overwrite-question))
;	  (#\ (draw-overwrite-on)))
	)
    (draw-goto-inside-matrix)))

(defun draw-left-and-funcall (&optional (func nil))
  (let* ((draw-auto-update *draw-auto-update*)
	 (*draw-auto-update* nil))
    (declare (special *draw-auto-update*))
    (when (draw-inside-matrix-p)
      (if (functionp func) (funcall func))
      (when (and (not (bolp))
		 (save-excursion (backward-char) (draw-inside-matrix-p)))
	(backward-char)
	(setq *draw-auto-update* draw-auto-update)
	(if (functionp func) (funcall func)))
      )))

(defun draw-right-and-funcall (&optional (func nil))
  (let* ((draw-auto-update *draw-auto-update*)
	 (*draw-auto-update* nil))
    (when (draw-inside-matrix-p)
      (if (functionp func) (funcall func))
      (when (and (not (eolp))
		 (save-excursion (forward-char) (draw-inside-matrix-p)))
	(forward-char)
	(setq *draw-auto-update* draw-auto-update)
	(if (functionp func) (funcall func)))
      )))

(defun draw-up-and-funcall (&optional (func nil))
  (let* ((draw-auto-update *draw-auto-update*)
	 (*draw-auto-update* nil))
    (when (draw-inside-matrix-p)
      (if (functionp func) (funcall func))
      (when (save-excursion (and (previous-line) (draw-inside-matrix-p)))
	(previous-line)
	(setq *draw-auto-update* draw-auto-update)
	(if (functionp func) (funcall func)))
      )))

(defun draw-down-and-funcall (&optional (func nil))
  (let* ((draw-auto-update *draw-auto-update*)
	 (*draw-auto-update* nil))
    (when (draw-inside-matrix-p)
      (if (functionp func) (funcall func))
      (when (save-excursion (and (next-line) (draw-inside-matrix-p)))
	(next-line)
	(setq *draw-auto-update* draw-auto-update)
	(if (functionp func) (funcall func)))
      )))

(defun draw-left ()
  (interactive)
  (draw-left-and-funcall nil))
(defun draw-right ()
  (interactive)
  (draw-right-and-funcall nil))
(defun draw-up ()
  (interactive)
  (draw-up-and-funcall nil))
(defun draw-down ()
  (interactive)
  (draw-down-and-funcall nil))

(defun draw-paint-left ()
  (interactive)
  (draw-left-and-funcall #'draw-overwrite-on))
(defun draw-paint-right ()
  (interactive)
  (draw-right-and-funcall #'draw-overwrite-on))
(defun draw-paint-up ()
  (interactive)
  (draw-up-and-funcall #'draw-overwrite-on))
(defun draw-paint-down ()
  (interactive)
  (draw-down-and-funcall #'draw-overwrite-on))

(defun draw-erase-left ()
  (interactive)
  (draw-left-and-funcall #'draw-overwrite-question))
(defun draw-erase-right ()
  (interactive)
  (draw-right-and-funcall #'draw-overwrite-question))
(defun draw-erase-up ()
  (interactive)
  (draw-up-and-funcall #'draw-overwrite-question))
(defun draw-erase-down ()
  (interactive)
  (draw-down-and-funcall #'draw-overwrite-question))

(defun draw-get-height ()
  (save-excursion
    (let (opoint height)
      (goto-char (point-min))
      (draw-goto-inside-matrix)
      (setq opoint (point))
      (setq height 0)
      (while (draw-inside-matrix-p)
	(incf height)
	(unless (next-line)
	  (return)))
      height)))

(defun draw-get-width ()
  (save-excursion
    (let (opoint width)
      (goto-char (point-min))
      (draw-goto-inside-matrix)
      (setq opoint (point))
      (setq width 0)
      (while (draw-inside-matrix-p)
	(incf width)
	(forward-char))
      width)))

(defun draw-set-input-data-from-matrix ()
  (save-excursion
    (let ((height (draw-get-height))
	  (width (draw-get-width))
	  opoint len
	  left-input upper-input line-input)
      (goto-char (point-min))
      (draw-goto-inside-matrix)
      (setq opoint (point))
      (dotimes (i height)
	(setq line-input nil)
	(setq len 0)
	(while (draw-inside-matrix-p)
	  (if (looking-for "")
	      (incf len)
	    (when (plusp len)
	      (push len line-input)
	      (setq len 0)))
	  (forward-char))
	(when (plusp len)
	  (push len line-input))
	(push (nreverse line-input) left-input)
	(goto-char opoint)
	(next-line (1+ i)))
      (setq left-input (nreverse left-input))
      (goto-char opoint)
      (dotimes (i width)
	(setq line-input nil)
	(setq len 0)
	(while (draw-inside-matrix-p)
	  (if (looking-for "")
	      (incf len)
	    (when (plusp len)
	      (push len line-input)
	      (setq len 0)))
	  (unless (next-line)
	    (return)))
	(when (plusp len)
	  (push len line-input))
	(push (nreverse line-input) upper-input)
	(goto-char opoint)
	(forward-char (1+ i)))
      (setq upper-input (nreverse upper-input))
      (setq input-data (list left-input upper-input))
      )))

(defun draw-get-matrix-simple-info-from-matrix ()
  (save-excursion
    (let ((height (draw-get-height))
	  (width (draw-get-width))
	  opoint
	  matrix-simple-info row-simple-info line-simple-info)
      (goto-char (point-min))
      (draw-goto-inside-matrix)
      (setq opoint (point))
      (setq matrix-simple-info (make-matrix-simple-info input-data))
      (setq row-simple-info (car matrix-simple-info))
      (dotimes (i height)
	(setq line-simple-info (nth i row-simple-info))
	(dotimes (j width)
	  (setf (car (nth j line-simple-info))
		(case (schar (buffer-substring (point) (1+ (point))) 0)
		  (#\ 1)
		  (#\~ 0)
		  (#\ nil)
		  (t nil)))
	  (forward-char))
	(goto-char opoint)
	(next-line (1+ i)))
      matrix-simple-info)))

(defun draw-get-current-matrix-row ()
  (save-excursion
    (let ((line-number (current-line-number)))
      (unless (draw-inside-matrix-p)
	(return-from draw-get-current-matrix-row nil))
      (goto-char (point-min))
      (draw-goto-inside-matrix)
      (- line-number (current-line-number)))))

(defun draw-get-current-matrix-col ()
  (save-excursion
    (let ((opoint (point)))
      (unless (draw-inside-matrix-p)
	(return-from draw-get-current-matrix-col nil))
      (goto-bol)
      (draw-goto-inside-matrix)
      (- opoint (point)))))

(defun draw-goto-matrix (row col)
  (goto-char (point-min))
  (draw-goto-inside-matrix)
  (forward-char col)
  (next-line row))

(defun draw-print-matrix-update ()
  (interactive)
  (let ((row (draw-get-current-matrix-row))
	(col (draw-get-current-matrix-col)))
    (draw-set-input-data-from-matrix)
    (draw-print-matrix input-data (draw-get-matrix-simple-info-from-matrix))
    (unless row (setq row 0))
    (unless col (setq col 0))
    (draw-goto-matrix row col)))

(defun draw-auto-update-toggle ()
  (interactive)
  (setq *draw-auto-update* (not *draw-auto-update*)))

(defun draw-get-input-data ()
  (save-excursion
    (if (find-buffer *buffer-draw*)
	(progn
	  (set-draw-buffer)
	  (draw-set-input-data-from-matrix)
	  input-data)
      nil)))

(defun draw-infoinput-save-data ()
  (interactive)
  (if (find-buffer *buffer-infoinput*)
      (progn
	(unless (= (count-windows) 2)
	  (delete-other-windows)
	  (split-window *infoinput-height*))
	(set-infoinput-buffer)
	(infoinput-save-data)
	t)
    nil))
