;;; -*- Mode: Lisp -*-
;;;
;;; ilogic/infoinput.l --- Ilust Logic for xyzzy

;;; Copyright (C) 2001 OHKUBO Hiroshi

;;; Author: OHKUBO Hiroshi
;;; Time-stamp: <2001/10/10 07:40:27 +0900>

;;; Code

(provide "ilogic/infoinput")

(in-package "ilogic")

;(eval-when (:compile-toplevel :load-toplevel :execute)
;  (require "ilogic/data"))

(defvar *info-class* "user")
(defvar *info-title* "No Title")
(defvar *info-problem* "")
(defvar *info-level* "")
(defvar *info-answer* "")
(defvar *info-comment* "")
(defvar *info-name* "nobody")
(defvar *info-email* "nobody@nowhere.org")
(defvar *info-url* "http://")
(defvar *info-height* "")
(defvar *info-width* "")

(defvar *info-color-header* 0)
(defvar *info-color-class* 0)
(defvar *info-color-title* 0)
(defvar *info-color-problem* 0)
(defvar *info-color-level* 0)
(defvar *info-color-answer* 0)
(defvar *info-color-comment* 0)
(defvar *info-color-date* 0)
(defvar *info-color-name* 0)
(defvar *info-color-email* 0)
(defvar *info-color-url* 0)
(defvar *info-color-height* 0)
(defvar *info-color-width* 0)

(defvar *infoinput-height* 10)
(defvar *infoinput-scroll-margin* 3)

(defvar *info-line* "--- ")

(defvar *buffer-infoinput* "I-Logic: InfoInput")

(defvar *infoinput-mode-hook* nil)
(defvar *infoinput-create-hook* nil)

(defvar *infoinput-map* nil)
(setq *infoinput-map* (make-sparse-keymap))
(define-key *infoinput-map* #\TAB 'infoinput-goto-next)
(define-key *infoinput-map* #\RET 'infoinput-return)
(define-key *infoinput-map* '(#\C-x #\C-s) 'infoinput-save-data)
(define-key *infoinput-map* '(#\C-c #\i) 'infoinput-input-print-matrix-base)
(define-key *infoinput-map* '(#\C-c #\d) 'infoinput-draw-print-matrix-base)
(define-key *infoinput-map* '(#\C-c #\q) 'infoinput-delete)

(defun set-infoinput-buffer ()
  (set-buffer (get-buffer-create *buffer-infoinput*))
  (infoinput-mode))

(defun infoinput-delete ()
  (interactive)
  (delete-buffer *buffer-infoinput*)
  (if (find-buffer *buffer-input*)
      (input-delete))
  (if (find-buffer *buffer-draw*)
      (draw-delete))
  (menu-show))

(defun infoinput ()
  (interactive)
  (set-infoinput-buffer)
  (infoinput-create))

(defun infoinput-mode ()
  (setq buffer-mode 'ilogic-infoinput)
  (setq mode-name *mode-name*)
  (make-local-variable 'title-bar-format)
  (setq title-bar-format (format nil "~A - %b" *version-name*))
  (make-local-variable '*scroll-margin*)
  (setq *scroll-margin* *infoinput-scroll-margin*)
  (setq need-not-save t)
  (setq auto-save nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-line-number* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-newline* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-eof* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-cursor-line* t)
  (set-buffer-fold-width nil)
  (use-keymap *infoinput-map*)
  (run-hooks '*infoinput-mode-hook*))

(defun infoinput-create ()
  (let ((kept-undo-information nil)
	start)
    (declare (special kept-undo-information))
    (erase-buffer (selected-buffer))

    (set-text-attribute (point)
			(progn
			  (insert "[j[ɖ߂]")
			  (point))
			'menu)
    (set-text-attribute (point)
			(progn
			  (insert "[̓]")
			  (point))
			'input)
    (set-text-attribute (point)
			(progn
			  (insert "[̍쐬]")
			  (point))
			'draw)
    (set-text-attribute (point)
			(progn
			  (insert "[̕ۑ]")
			  (point))
			'save)
    (insert "\n")
    (insert "-" 78)
    (insert "\n")
    (set-text-attribute (point)
			(progn
			  (insert "    F ")
			  (point))
			'header-class
			:foreground *info-color-header*
			)
    (setq start (point))
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-class*))
			  (point))
			'class
			:foreground *info-color-class*
			)
    (set-text-attribute (point)
			(progn
			  (insert "^CgF ")
			  (point))
			'header-title
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-title*))
			  (point))
			'title
			:foreground *info-color-title*
			)
    (set-text-attribute (point)
			(progn
			  (insert "蕶  F ")
			  (point))
			'header-problem
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-problem*))
			  (point))
			'problem
			:foreground *info-color-problem*
			)
    (set-text-attribute (point)
			(progn
			  (insert "x  F ")
			  (point))
			'header-level
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-level*))
			  (point))
			'level
			:foreground *info-color-level*
			)
    (set-text-attribute (point)
			(progn
			  (insert "    F ")
			  (point))
			'header-answer
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-answer*))
			  (point))
			'answer
			:foreground *info-color-answer*
			)
    (set-text-attribute (point)
			(progn
			  (insert "RgF ")
			  (point))
			'header-comment
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-comment*))
			  (point))
			'comment
			:foreground *info-color-comment*
			)
    (insert (concat *info-line* "\n"))

    (set-text-attribute (point)
			(progn
			  (insert "    F ")
			  (point))
			'header-height
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-height*))
			  (point))
			'height
			:foreground *info-color-height*
			)
    (set-text-attribute (point)
			(progn
			  (insert "      F ")
			  (point))
			'header-width
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-width*))
			  (point))
			'width
			:foreground *info-color-width*
			)
    (insert (concat *info-line* "\n"))

    (set-text-attribute (point)
			(progn
			  (insert "O    F ")
			  (point))
			'header-name
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-name*))
			  (point))
			'name
			:foreground *info-color-name*
			)
    (set-text-attribute (point)
			(progn
			  (insert "E-Mail  F ")
			  (point))
			'header-email
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-email*))
			  (point))
			'email
			:foreground *info-color-email*
			)
    (set-text-attribute (point)
			(progn
			  (insert "URL     F ")
			  (point))
			'header-url
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" *info-url*))
			  (point))
			'url
			:foreground *info-color-url*
			)
    (insert (concat *info-line* "\n"))
    (set-buffer-modified-p nil)

    (goto-char start)
    (run-hooks '*infoinput-create-hook*)
    ))

(defun infoinput-check-info ()
  (interactive)
  (let ((title (infoinput-get-title))
	(height (infoinput-get-height))
	(width (infoinput-get-width)))
    (cond
     ((not (plusp height))
      (message "uvsłB")
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'height)
	(if from (goto-char from)))
      nil)
     ((not (plusp width))
      (message "uvsłB")
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'width)
	(if from (goto-char from)))
      nil)
     ((string-equal "" title)
      (message "u^Cgv͂Ă܂B")
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'title)
	(if from (goto-char from)))
      nil)
     (t t))))

(defun infoinput-input-print-matrix-base ()
  (interactive)
  (let ((height (infoinput-get-height))
	(width (infoinput-get-width)))
    (when (or (input-get-input-data) (draw-get-input-data))
      (if (yes-or-no-p "͓r̃f[^j܂H")
	  (progn
	    (if (find-buffer *buffer-input*)
		(delete-buffer *buffer-input*))
	    (if (find-buffer *buffer-draw*)
		(delete-buffer *buffer-draw*))
	    (delete-other-windows))
	(progn
	  (unless (= (count-windows) 2)
	    (delete-other-windows)
	    (split-window *infoinput-height*))
	  (other-window)
	  (cond
	   ((input-get-input-data)
	    (set-input-buffer))
	   ((draw-get-input-data)
	    (set-draw-buffer)))
	  (return-from infoinput-input-print-matrix-base nil))))
    (if (infoinput-check-info)
	(progn
	  (unless (= (count-windows) 2)
	    (delete-other-windows)
	    (split-window *infoinput-height*))
	  (other-window)
	  (set-input-buffer)
	  (input-print-matrix-base height width)
	  (input-goto-next-row-input)
	  t)
      nil)
    ))

(defun infoinput-draw-print-matrix-base ()
  (interactive)
  (let ((height (infoinput-get-height))
	(width (infoinput-get-width)))
    (when (or (input-get-input-data) (draw-get-input-data))
      (if (yes-or-no-p "͓r̃f[^j܂H")
	  (progn
	    (if (find-buffer *buffer-input*)
		(delete-buffer *buffer-input*))
	    (if (find-buffer *buffer-draw*)
		(delete-buffer *buffer-draw*))
	    (delete-other-windows))
	(progn
	  (unless (= (count-windows) 2)
	    (delete-other-windows)
	    (split-window *infoinput-height*))
	  (other-window)
	  (cond
	   ((input-get-input-data)
	    (set-input-buffer))
	   ((draw-get-input-data)
	    (set-draw-buffer)))
	  (return-from infoinput-draw-print-matrix-base nil))))
    (if (infoinput-check-info)
	(progn
	  (unless (= (count-windows) 2)
	    (delete-other-windows)
	    (split-window *infoinput-height*))
	  (other-window)
	  (set-draw-buffer)
	  (draw-print-matrix-base height width)
	  (set-buffer-modified-p nil)
	  t)
      nil)
    ))

(defun infoinput-get-any (tag)
  (save-excursion
    (let ((current-buf (selected-buffer))
	  (infoinput-buf (find-buffer *buffer-infoinput*))
	  return-string)
      (unless infoinput-buf
	(setq return-string "")
	(return-from infoinput-get-any return-string))
      (set-buffer infoinput-buf)
      (multiple-value-bind (from to tag)
	  (find-text-attribute tag)
	(if (not from)
	    (setq return-string "")
	  (setq return-string
		(buffer-substring (progn (goto-char from) (point))
				  (progn (goto-eol) (point))))))
      (setq return-string (string-trim " " return-string))
      (set-buffer current-buf)
      return-string)))
	
(defun infoinput-get-class ()
  (infoinput-get-any 'class))
(defun infoinput-get-title ()
  (infoinput-get-any 'title))
(defun infoinput-get-problem ()
  (infoinput-get-any 'problem))
(defun infoinput-get-level ()
  (infoinput-get-any 'level))
(defun infoinput-get-answer ()
  (infoinput-get-any 'answer))
(defun infoinput-get-comment ()
  (infoinput-get-any 'comment))

(defun infoinput-get-height ()
  (let ((str (infoinput-get-any 'height)))
    (if (string-match "[1-9][0-9]*" str)
	(parse-integer (substring str (match-beginning 0) (match-end 0)))
      0)))
(defun infoinput-get-width ()
  (let ((str (infoinput-get-any 'width)))
    (if (string-match "[1-9][0-9]*" str)
	(parse-integer (substring str (match-beginning 0) (match-end 0)))
      0)))
  
(defun infoinput-get-name ()
  (infoinput-get-any 'name))
(defun infoinput-get-email ()
  (infoinput-get-any 'email))
(defun infoinput-get-url ()
  (infoinput-get-any 'url))

(defun equal-symbol-prefix-p (prefix-string symbol)
  (let ((len (length prefix-string))
	(symbol-string (symbol-name symbol)))
    (if (< (length symbol-string) len)
	nil
      (string= prefix-string (substring (symbol-name symbol) 0 len)))
    ))

(defun infoinput-goto-next ()
  (interactive)
  (multiple-value-bind (from to tag)
      (find-text-attribute "header-" :start (point) :test #'equal-symbol-prefix-p)
    (if (integerp to)
      (goto-char to)
      (progn
	(goto-char (point-min))
	(multiple-value-bind (from to tag)
	    (find-text-attribute "header-" :start (point) :test #'equal-symbol-prefix-p)
	  (if (integerp to)
	      (goto-char to)))))))

(defun infoinput-delete-by-tag ()
  (save-excursion
    (when (scan-buffer "[\n[]" :regexp t :reverse t)
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'menu :start (point) :end (1+ (point)))
	(if from
	    (progn
	      (infoinput-delete)
	      t)
	  nil)))))


(defun infoinput-save-data-by-tag ()
  (let ((opoint (point)))
    (when (scan-buffer "[\n[]" :regexp t :reverse t)
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'save :start (point) :end (1+ (point)))
	(if from
	    (progn
	      (infoinput-save-data)
	      t)
	  (progn
	    (goto-char opoint)
	    nil))
	))))

(defun infoinput-input-print-matrix-base-by-tag ()
  (let ((opoint (point)))
    (when (scan-buffer "[\n[]" :regexp t :reverse t)
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'input :start (point) :end (1+ (point)))
	(if from
	    (progn
	      (infoinput-input-print-matrix-base)
	      t)
	  (progn
	    (goto-char opoint)
	    nil))
	))))

(defun infoinput-draw-print-matrix-base-by-tag ()
  (let ((opoint (point)))
    (when (scan-buffer "[\n[]" :regexp t :reverse t)
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'draw :start (point) :end (1+ (point)))
	(if from
	    (progn
	      (infoinput-draw-print-matrix-base)
	      t)
	  (progn
	    (goto-char opoint)
	    nil))
	))))

(defun infoinput-return ()
  (interactive)
  (cond
   ((infoinput-delete-by-tag))
   ((infoinput-save-data-by-tag))
   ((infoinput-input-print-matrix-base-by-tag))
   ((infoinput-draw-print-matrix-base-by-tag))
   ((infoinput-goto-next))))

(defun infoinput-save-data ()
  (interactive)
  (let (class-directory
	file-name
	ilogicdata
	ilogicdata-file
	input-data)
    (setq class-directory
	  (merge-pathnames (delete-unsuitable-filename-char (infoinput-get-class))
			   *data-directory*))
    (unless (file-exist-p class-directory)
      (create-directory class-directory))
    (setq file-name (get-new-file-name class-directory))
    (setq ilogicdata-file (merge-pathnames file-name class-directory))

    (when (string-equal "" (infoinput-get-title))
      (message "u^Cgv͂Ă܂B")
      (goto-char (point-min))
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'title)
	(if from (goto-char from)))
      (return-from infoinput-save-data nil))

    (setq input-data
	  (cond
	   ((input-get-input-data))
	   ((draw-get-input-data))
	   (t nil)))
    (unless input-data
      (message "f[^͂Ă܂B")
      (infoinput-input-print-matrix-base)
      (return-from infoinput-save-data nil))
    
    (with-open-file (s ilogicdata-file
		       :direction :output
		       :if-does-not-exist :create)
      (setq ilogicdata
	    (make-ilogicdata :class   (infoinput-get-class)
			     :title   (infoinput-get-title)
			     :problem (infoinput-get-problem)
			     :level   (infoinput-get-level)
			     :answer  (infoinput-get-answer)
			     :comment (infoinput-get-comment)
			     :input   input-data
;			     :width   (infoinput-get-width)
;			     :height  (infoinput-get-height)
			     :width   (get-input-width input-data)
			     :height  (get-input-height input-data)
			     :date    (get-universal-time)
			     :author (make-author
				      :name  (infoinput-get-name)
				      :email (infoinput-get-email)
				      :url   (infoinput-get-url)
				      )
			     :autosolve nil
			     ))
      (format s "~S\n" ilogicdata))
    (if (yes-or-no-p "̖̓́^쐬s܂H")
	(progn
	  (infoinput-delete)
	  (infoinput))
      (infoinput-delete))
    ))

