;;; -*- Mode: Lisp -*-
;;;
;;; ilogic/infoupdate.l --- Ilust Logic for xyzzy

;;; Copyright (C) 2001 OHKUBO Hiroshi

;;; Author: OHKUBO Hiroshi
;;; Time-stamp: <2001/10/10 07:46:59 +0900>

;;; Code

(provide "ilogic/infoupdate")

(in-package "ilogic")

;(eval-when (:compile-toplevel :load-toplevel :execute)
;  (require "ilogic/data"))

(defvar *buffer-infoupdate* "I-Logic: InfoUpdate")

(defvar *infoupdate-mode-hook* nil)

(defvar *infoupdate-map* nil)
(setq *infoupdate-map* (make-sparse-keymap))

(defun set-infoupdate-buffer ()
  (set-buffer (get-buffer-create *buffer-infoupdate*))
  (infoupdate-mode))

(defun infoupdate-delete ()
  (interactive)
  (delete-buffer *buffer-infoupdate*))

(defun infoupdate-mode ()
  (setq buffer-mode 'ilogic-infoupdate)
  (setq mode-name *mode-name*)
  (make-local-variable 'title-bar-format)
  (setq title-bar-format (format nil "~A - %b" *version-name*))
  (setq buffer-read-only t)
  (setq need-not-save t)
  (setq auto-save nil)
  (toggle-ime nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-line-number* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-newline* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-eof* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-cursor-line* nil)
  (set-buffer-fold-width nil)
  (use-keymap *infoupdate-map*)
  (run-hooks '*infoupdate-mode-hook*))


(defun infoupdate-show (ilogicdata)
  (let ((buffer-read-only nil)
	(kept-undo-information nil)
	author
	autosolve)
    (declare (special buffer-read-only))
    (declare (special kept-undo-information))
    (erase-buffer (selected-buffer))

    (set-text-attribute (point)
			(progn
			  (insert "    F ")
			  (point))
			'header-class
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (ilogicdata-class ilogicdata)))
			  (point))
			'class
			:foreground *info-color-class*
			)

    (set-text-attribute (point)
			(progn
			  (insert "^CgF ")
			  (point))
			'header-title
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (ilogicdata-title ilogicdata)))
			  (point))
			'title
			:foreground *info-color-title*
			)
    
    (set-text-attribute (point)
			(progn
			  (insert "蕶  F ")
			  (point))
			'header-problem
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (ilogicdata-problem ilogicdata)))
			  (point))
			'problem
			:foreground *info-color-problem*
			)

    (set-text-attribute (point)
			(progn
			  (insert "x  F ")
			  (point))
			'header-level
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (ilogicdata-level ilogicdata)))
			  (point))
			'level
			:foreground *info-color-level*
			)

    (set-text-attribute (point)
			(progn
			  (insert "    F ")
			  (point))
			'header-answer
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (ilogicdata-answer ilogicdata)))
			  (point))
			'answer
			:foreground *info-color-answer*
			)

    (set-text-attribute (point)
			(progn
			  (insert "RgF ")
			  (point))
			'header-comment
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (ilogicdata-comment ilogicdata)))
			  (point))
			'comment
			:foreground *info-color-comment*
			)

    (set-text-attribute (point)
			(progn
			  (insert "쐬  F ")
			  (point))
			'header-date
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert
			   (if	(integerp (ilogicdata-date ilogicdata))
			       (format nil "~A~%"
				       (format-date-string "%Y/%m/%d %H:%M:%S"
							   (ilogicdata-date ilogicdata)))
			     (format nil "~A~%" (ilogicdata-date ilogicdata))))
			  (point))
			'date
			:foreground *info-color-date*
			)

    (insert (concat *info-line* "\n"))

    (set-text-attribute (point)
			(progn
			  (insert "    F ")
			  (point))
			'header-height
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (ilogicdata-height ilogicdata)))
			  (point))
			'height
			:foreground *info-color-height*
			)

    (set-text-attribute (point)
			(progn
			  (insert "      F ")
			  (point))
			'header-width
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (ilogicdata-width ilogicdata)))
			  (point))
			'width
			:foreground *info-color-width*
			)

    (insert (concat *info-line* "\n"))

    (setq author (ilogicdata-author ilogicdata))

    (set-text-attribute (point)
			(progn
			  (insert "O    F ")
			  (point))
			'header-name
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (author-name author)))
			  (point))
			'name
			:foreground *info-color-name*
			)

    (set-text-attribute (point)
			(progn
			  (insert "E-Mail  F ")
			  (point))
			'header-email
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (author-email author)))
			  (point))
			'email
			:foreground *info-color-email*
			)

    (set-text-attribute (point)
			(progn
			  (insert "URL     F ")
			  (point))
			'header-url
			:foreground *info-color-header*
			)
    (set-text-attribute (point)
			(progn
			  (insert (format nil "~A~%" (author-url author)))
			  (point))
			'url
			:foreground *info-color-url*
			)

    (insert (concat *info-line* "\n"))

    (set-buffer-modified-p nil)
    (goto-char (point-min))
    ))

