;;; -*- Mode: Lisp -*-
;;;
;;; ilogic/input.l --- Ilust Logic for xyzzy

;;; Copyright (C) 2001 OHKUBO Hiroshi

;;; Author: OHKUBO Hiroshi
;;; Time-stamp: <2001/10/08 09:44:39 +0900>

;;; Code

(provide "ilogic/input")

(in-package "ilogic")

;(eval-when (:compile-toplevel :load-toplevel :execute)
;  (require "ilogic/data"))

(defvar *buffer-input* "I-Logic: Input")

(defvar *input-mode-hook* nil)
(defvar *input-create-hook* nil)

(defvar *input-map* nil)
(setq *input-map* (make-sparse-keymap))
(define-key *input-map* #\n 'input-goto-next-input)
(define-key *input-map* #\RET 'input-update)
(define-key *input-map* #\SPC 'input-goto-next-input)
(define-key *input-map* #\TAB 'input-goto-next-input)
(define-key *input-map* '(#\C-x #\C-s) 'input-infoinput-save-data)
(define-key *input-map* '(#\C-c #\q) 'input-delete)

(defun set-input-buffer ()
  (set-buffer (get-buffer-create *buffer-input*))
  (input-mode))

(defun input-mode ()
  (setq buffer-mode 'ilogic-input)
  (setq mode-name *mode-name*)
  (make-local-variable 'input-data)
  (make-local-variable 'title-bar-format)
  (setq title-bar-format (format nil "~A - %b" *version-name*))
  (setq buffer-read-only t)
  (setq need-not-save t)
  (setq kept-undo-information nil)
  (setq auto-save nil)
  (toggle-ime nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-line-number* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-newline* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-eof* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-cursor-line* t)
  (set-buffer-fold-width nil)
  (use-keymap *input-map*)
  (run-hooks '*input-mode-hook*))

(defun input-delete ()
  (interactive)
  (delete-buffer *buffer-input*)
  (if (find-buffer *buffer-infoinput*)
      (infoinput-delete))
  (menu-show))

(defun input-goto-next-row-input ()
  (interactive)
  (multiple-value-bind (from to tag)
      (find-text-attribute 'row-input :start (1+ (point)) :end (point-max))
    (if from
	(goto-char from)
      nil)))

(defun input-goto-next-col-input ()
  (interactive)
  (multiple-value-bind (from to tag)
      (find-text-attribute 'col-input :start (1+ (point)) :end (point-max))
    (if from
	(goto-char from)
      nil)))

(defun input-goto-next-input ()
  (interactive)
  (unless (input-goto-next-col-input)
    (unless (input-goto-next-row-input)
      (goto-char (point-min))
      (input-goto-next-col-input))))

(defun check-numbers (numbers)
  (let ((in (make-string-input-stream numbers))
	number-list num)

    (while (setq num (read in nil))
      (cond
       ((or (not (integerp num)) (not (plusp num)))
	(print "̐͂Ă")
	(setq number-list nil)
	(return))
       (t
	(push num number-list))))

    (reverse number-list)))

(defun input-numbers ()
  (interactive)
  (let (input)
    (unless
	(or (find-text-attribute 'col-input :start (point) :end (1+ (point)))
	    (find-text-attribute 'row-input :start (point) :end (1+ (point))))
      (return-from input-numbers nil))

    (loop
      (setq input (check-numbers (read-string "Input Numbers: ")))
      (when input
	(return input)))))

(defun input-print-matrix-base (height width)
  (let (matrix-simple-info)
    (setq input-data (make-sparse-input height width))
    (setq matrix-simple-info (make-matrix-simple-info input-data))
    (input-set-lattice-matrix-simple-info matrix-simple-info)
    (input-print-matrix input-data matrix-simple-info)
    (goto-char (point-min))
    ))

(defun input-set-lattice-matrix-simple-info (matrix-simple-info &optional (output 0) (interval 5))
  (let ((height (length (car matrix-simple-info)))
	(width (length (cadr matrix-simple-info)))
	line-simple-info)
    (dotimes (i height)
      (when (zerop (mod (1+ i) interval))
	(setq line-simple-info (nth i (car matrix-simple-info)))
	(dolist (cell-simple-info line-simple-info)
	  (setf (car cell-simple-info) output))))
    (dotimes (i width)
      (when (zerop (mod (1+ i) interval))
	(setq line-simple-info (nth i (cadr matrix-simple-info)))
	(dolist (cell-simple-info line-simple-info)
	  (setf (car cell-simple-info) output))))
    ))
    
(defun make-sparse-input (height width)
  (let (left-input upper-input)
    (dotimes (i height)
      (push nil left-input)
      )
    (dotimes (i width)
      (push nil upper-input)
      )
    (list left-input upper-input)))

(defun input-print-matrix (input &optional matrix-simple-info)
  (let ((buffer-read-only nil)
	(indication (make-indication input))
	(height (get-input-height input))
	(width (get-input-width input))
	base-column base-line)
    (declare (special buffer-read-only))
    (erase-buffer (selected-buffer))
    (print-matrix-base indication matrix-simple-info nil)
    (goto-char (point-min))
    (scan-buffer "[~]" :regexp t)
    (setq base-line (current-line-number))
    (setq base-column (current-column))
    
    (goto-char (point-min))
    (when (= 1 (current-line-number))
      (insert " " (+ (1- base-column) (* width 2)))
      (insert "\n"))
    (goto-char (point-min))
    (when (= 0 (current-column))
      (while (looking-for " ")
	(insert " ")
	(next-line)
	(goto-bol))
      (dotimes (i height)
	(insert " ")
	(next-line)
	(goto-bol))
      (insert " "))

    (goto-char (point-min))
    (scan-buffer "[~]" :regexp t)
    (setq base-line (current-line-number))
    (setq base-column (current-column))

    (previous-line)
    (backward-char)
    (dotimes (i width)
      (forward-char 2)
      (set-text-attribute (point) (1+ (point)) 'col-input))
    (next-line)
    (goto-bol)
    (goto-column base-column)
    (backward-char)
    (dotimes (i height)
      (set-text-attribute (point) (1+ (point)) 'row-input)
      (next-line))
    ))

(defun input-get-current-matrix-row ()
  (save-excursion
    (let (line-number
	  base-line-number
	  row)
      (setq line-number (current-line-number))
      (goto-char (point-min))
      (unless (scan-buffer "[~]" :regexp t)
	(return-from input-get-current-matrix-row nil))
      (setq base-line-number (current-line-number))
      (setq row (- line-number base-line-number))
      row)))


(defun input-get-current-matrix-col ()
  (save-excursion
    (let (column
	  base-column
	  col)
      (setq column (current-column))
      (goto-char (point-min))
      (unless (scan-buffer "[~]" :regexp t)
	(return-from input-get-current-matrix-col nil))
      (setq base-column (current-column))
      (setq col (floor (/ (- column base-column) 2)))
      col)))

(defun input-goto-matrix (row col)
  (goto-char (point-min))
  (unless (scan-buffer "[~]" :regexp t)
    (return-from input-goto-matrix nil))
  (forward-char col)
  (next-line row)
  (if (minusp row)
      (forward-char))
  )

(defun input-update ()
  (interactive)
  (let (input
	row col
	matrix-simple-info)
    (setq input (input-numbers))
    (unless input
      (return-from input-update nil))
    (setq row (input-get-current-matrix-row))
    (setq col (input-get-current-matrix-col))

    (if (find-text-attribute 'row-input :start (point) :end (1+ (point)))
	(progn
	  (setf (nth row (car input-data)) input)
	  )
      (progn
	(setf (nth col (cadr input-data)) input)
	))
    (setq matrix-simple-info (make-matrix-simple-info input-data))
    (input-set-lattice-matrix-simple-info matrix-simple-info)
    (input-print-matrix input-data matrix-simple-info)
    (input-goto-matrix row col)
    (input-goto-next-input)
    ))

(defun input-get-input-data ()
  (save-excursion
    (if (find-buffer *buffer-input*)
	(progn
	  (set-input-buffer)
	  input-data)
      nil)))

(defun input-infoinput-save-data ()
  (interactive)
  (if (find-buffer *buffer-infoinput*)
      (progn
	(unless (= (count-windows) 2)
	  (delete-other-windows)
	  (split-window *infoinput-height*))
	(set-infoinput-buffer)
	(infoinput-save-data)
	t)
    nil))

