;;; -*- Mode: Lisp -*-
;;;
;;; ilogic/main.l --- Ilust Logic for xyzzy

;;; Copyright (C) 2001 OHKUBO Hiroshi

;;; Author: OHKUBO Hiroshi
;;; Version: 0.0.0.4
;;; Time-stamp: <2001/10/10 08:11:35 +0900>

;;; install
;;;

;;; key-bind
;;;

;;; changes
;;;


;;; Code

(defpackage "ilogic"
  (:use "lisp" "editor"))

(provide "ilogic/main")

;(eval-when (:compile-toplevel :load-toplevel :execute)
;  (require "ilogic/data"))

(require "ilogic/data")
(require "ilogic/etc")
(require "ilogic/method")
(require "ilogic/menu")
(require "ilogic/infoinput")
(require "ilogic/input")
(require "ilogic/draw")
(require "ilogic/select")
(require "ilogic/infoupdate")
(require "ilogic/autosolve")
(require "ilogic/popup")

(defun ilogic ()
  (interactive)
  (ilogic::start))

(defun ilogic-toggle ()
  (interactive)
  (ilogic::toggle))

(in-package "ilogic")

(defconstant *version* "0.0.0.4")

(defvar *version-name*
  (format nil "I-Logic-~A" *version*))

(defvar *mode-name* "I-Logic")
(defvar *pframe-name* "I-Logic")

(defvar *buffer-temp* "I-Logic: Temp")

(defvar *base-directory* "~/.ilogic")

(defvar *config-file-name* "config.l")
(defvar *config-file*
  (merge-pathnames *config-file-name* *base-directory*))

(defvar *data-directory-name* "data")
(defvar *data-directory*
  (merge-pathnames *data-directory-name* *base-directory*))

(defvar *ilogic-start-hook* nil)
(defvar *ilogic-exit-hook* nil)

(defvar *ilogic-running* nil)

(defvar *debug* nil)

(defun set-temp-buffer ()
  (set-buffer (get-buffer-create *buffer-temp*))
  (setq need-not-save t)
  (erase-buffer (selected-buffer)))

(defun init ()
  (unless (file-exist-p *base-directory*)
    (create-directory *base-directory*))
  (setq *config-file*
	(merge-pathnames *config-file-name* *base-directory*))
  (let ((lc (concat *config-file* "c")))
    (if (file-exist-p lc)
	(load lc)
      (when (file-exist-p *config-file*)
	(load *config-file*))))
  )
  
(defun start ()
  (init)
  (when (frame-get)
    (restore)
    (return-from start))
  (message "I-Logic: starting ...")
  (new-pseudo-frame *pframe-name*)
  (setq *ilogic-running* t)
  (menu-show)
  (message "I-Logic: starting ... done.")
  (run-hooks '*ilogic-start-hook*))

(defun exit ()
  (interactive)
  (unless *ilogic-running*
    (return-from exit))
  (let ((frame (frame-get)))
    (dolist (buf (buffer-list))
      (let ((bufname (buffer-name buf)))
	(when (string-match "^I-Logic:" bufname)
	  (delete-buffer bufname))))
    (when frame
      (delete-pseudo-frame frame))
    (setq *ilogic-running* nil)
    (run-hooks '*ilogic-exit-hook*)))

(defun hide ()
  (interactive)
  (next-pseudo-frame))

(defun frame-get ()
  (find-pseudo-frame *pframe-name*))

(defun restore ()
  (interactive)
  (let ((frame (frame-get)))
    (if frame
	(select-pseudo-frame frame)
      (start))))

(defun toggle ()
  (interactive)
  (if (eq *pframe-name*
	  (ed::pseudo-frame-name ed::*current-pseudo-frame*))
      (hide)
    (restore)))

(defvar *ilogic-mouse-left-press* '(
				    ))
(defun ilogic-mouse-left-press ()
  (interactive)
  (let ((owindow (selected-window))
	func)
    (set-window *last-mouse-window*)
    (continue-pre-selection)
    (if (setq func (cdr (assoc buffer-mode *ilogic-mouse-left-press*)))
	(progn
	  (goto-last-mouse-point)
	  (when (or *ilogic-mouse-single-click*
		    (> *last-mouse-click-count* 1))
	    (funcall func))
	  t)
      (progn
	(set-window owindow)
	(mouse-left-press)))))

(defun file-backup (file)
  (when (file-exist-p file)
    (copy-file file
	       (concat file ".bak")
	       :if-exists :overwrite)))

(add-hook 'ed::*kill-xyzzy-hook* 'exit)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; debug
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defvar *buffer-debug* "I-Logic: Debug")
(defvar *debug-stream* nil)
(defvar *debug-window-ratio* 1/8)

(defun debug (&optional (on nil))
  (if (or (eq *debug* on) (and *debug* on))
      (return-from debug))
  (if on
      (progn
	(setq *debug* on)
	(unless (find-buffer *buffer-debug*)
	  (let (buffer debug-buffer)
;           (toggle-trace-on-error)
	    (setq debug-buffer (get-buffer-create *buffer-debug*))
	    (setq buffer (selected-buffer))
	    (set-buffer debug-buffer)
;	    (kill-all-local-variables)
	    (make-local-variable 'need-not-save)
	    (setq need-not-save t)
	    (make-local-variable 'auto-save)
	    (setq auto-save nil)
;	    (make-local-variable 'kept-undo-information)
;	    (setq kept-undo-information nil)
	    (set-buffer-fold-width nil)
	    (set-local-window-flags (selected-buffer) *window-flag-cursor-line* t)
	    (set-local-window-flags (selected-buffer) *window-flag-ruler* t)
	    (setq *debug-stream* (make-buffer-stream debug-buffer))
	    (set-buffer buffer)
;	    (popup-debug-buffer)
	    ))))
  )

(defun popup-debug-buffer ()
  (pop-to-buffer (get-buffer-create *buffer-debug*)
    (truncate (* screen-height) *debug-window-ratio*)))
  
(defun debug-print (control-string &rest arguments)
  (when *debug*
    (buffer-stream-set-point *debug-stream* (point-min))
    (apply #'format (append `(,*debug-stream* ,control-string) arguments))
    ))
