;;; -*- Mode: Lisp -*-
;;;
;;; ilogic/menu.l --- Ilust Logic for xyzzy

;;; Copyright (C) 2001 OHKUBO Hiroshi

;;; Author: OHKUBO Hiroshi
;;; Time-stamp: <2001/10/10 06:39:41 +0900>

;;; Code

(provide "ilogic/menu")

(in-package "ilogic")

;(eval-when (:compile-toplevel :load-toplevel :execute)
;  (require "ilogic/data"))

(defvar *buffer-menu* "I-Logic: Menu")

(defvar *menu-mode-hook* nil)
(defvar *menu-show-hook* nil)


(defvar *menu-map* nil)
(setq *menu-map* (make-sparse-keymap))
(define-key *menu-map* #\RET 'menu-select)
(define-key *menu-map* #\LBtnUp 'ilogic-mouse-left-press)
(define-key *menu-map* #\RBtnUp 'ilogic-mouse-right-press)
(define-key *menu-map* #\n 'menu-goto-next-choice)
(define-key *menu-map* #\p 'menu-goto-previous-choice)
(define-key *menu-map* #\C-n 'menu-goto-next-choice)
(define-key *menu-map* #\C-p 'menu-goto-previous-choice)
(define-key *menu-map* #\Down 'menu-goto-next-choice)
(define-key *menu-map* #\Up 'menu-goto-previous-choice)
(define-key *menu-map* #\Right 'menu-goto-next-choice)
(define-key *menu-map* #\Left 'menu-goto-previous-choice)
(define-key *menu-map* #\SPC 'menu-goto-next-choice)
(define-key *menu-map* #\TAB 'menu-goto-next-choice)
(define-key *menu-map* #\q 'exit)
(define-key *menu-map* #\Q 'exit)

(defun set-menu-buffer ()
  (set-buffer (get-buffer-create *buffer-menu*))
  (menu-mode))

(defun menu-mode-line ()
;  (setq mode-line-format "")
  (setq title-bar-format (format nil "~A - %b" *version-name*)))

(defun menu-mode-line-update ()
  (save-excursion
    (set-buffer *buffer-menu*)
    (menu-mode-line)))

(defun menu-mode ()
  (setq buffer-mode 'ilogic-menu)
  (setq mode-name *mode-name*)
  (make-local-variable 'mode-line-format)
  (make-local-variable 'title-bar-format)
  (menu-mode-line)
  (setq buffer-read-only t)
  (setq need-not-save t)
  (setq kept-undo-information nil)
  (setq auto-save nil)
  (toggle-ime nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-line-number* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-newline* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-eof* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-cursor-line* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-vscroll-bar* nil)
  (set-buffer-fold-width nil)
  (use-keymap *menu-map*)
  (run-hooks '*menu-mode-hook*))

(defun menu-show ()
  (delete-other-windows)
  (if (find-buffer *buffer-menu*)
      (set-buffer *buffer-menu*)
    (progn
      (set-menu-buffer)
      (let ((current (current-line-number))
	    (buffer-read-only nil))
	(declare (special buffer-read-only))
	(erase-buffer (selected-buffer))
	(menu-insert-text)
	(menu-goto-next-choice)
	)))
  (run-hooks '*menu-show-hook*))

(defun menu-insert-text ()
  (insert "
  
  
  
  
  
  
  
  
  
  
  ~~~~~~~~~~~~~~~~~~~~~~
  ~~~~~~~~~~~~~~~~~~~~~
  ~~~~~~~~~~~~~~~~~~~~
  ~~~~~~~~~~~~
  ~~~~~~~~~~~~~~
  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



      1: ̑I
      2: ̓́^쐬

      q: I-Logic ̏I


  (SPACE Ńj[IAEnter Ō)
")
  (set-buffer-modified-p nil))

(defun menu-goto-next-choice ()
  (interactive)
  (let (opoint)
    (delete-text-attributes 'selected-menu)
    (unless (scan-buffer "\\([0-9a-zA-Z]+\\): \\(.+\\)$" :regexp t :no-dup t)
      (goto-char (point-min))
      (scan-buffer "\\([0-9a-zA-Z]+\\): \\(.+\\)$" :regexp t :no-dup t))
    (setq opoint (point))

    (when (string-match "\\([0-9a-zA-Z]+\\): \\(.+\\)$"
			(buffer-substring opoint (point-max)))
      (goto-char (+ opoint (match-beginning 1)))
      (set-text-attribute (+ opoint (match-beginning 2))
			  (+ opoint (match-end 2))
			  'selected-menu
			  :foreground 0
			  :background 1
			  :bold nil
			  :underline t)
      )))

(defun menu-goto-previous-choice ()
  (interactive)
  (let (opoint)
    (delete-text-attributes 'selected-menu)
    (unless (scan-buffer "\\([0-9a-zA-Z]\\): \\(.+\\)$" :regexp t :no-dup t :reverse t)
      (goto-char (point-max))
      (scan-buffer "\\([0-9a-zA-Z]+\\): \\(.+\\)$" :regexp t :no-dup t :reverse t))
    (setq opoint (point))

    (when (string-match "\\([0-9a-zA-Z]\\): \\(.+\\)$"
			(buffer-substring opoint (point-max)))
      (goto-char (+ opoint (match-beginning 1)))
      (set-text-attribute (+ opoint (match-beginning 2))
			  (+ opoint (match-end 2))
			  'selected-menu
			  :foreground 0
			  :background 1
			  :bold nil
			  :underline t)
      )))


(defun menu-select ()
  (interactive)
  (multiple-value-bind (from to tag)
      (find-text-attribute 'selected-menu)
    (goto-char from)
    (if	(scan-buffer "[0-9a-zA-Z]: " :regexp t :no-dup t :reverse t)
	(case (schar (buffer-substring (point) (1+ (point))) 0)
	  (#\1 (select))
	  (#\2 (infoinput))
	  (#\q (exit)))
      nil)))
