;;; -*- Mode: Lisp -*-
;;;
;;; ilogic/select.l --- Ilust Logic for xyzzy

;;; Copyright (C) 2001 OHKUBO Hiroshi

;;; Author: OHKUBO Hiroshi
;;; Time-stamp: <2001/10/10 06:34:55 +0900>

;;; Code

(provide "ilogic/select")

(in-package "ilogic")

(eval-when (:compile-toplevel :load-toplevel :execute)
  (require "ilogic/data"))

(defvar *buffer-select* "I-Logic: Select")

;(defvar *list-file-name* "list.txt")

(defvar *select-mode-hook* nil)

(defvar *select-height* 5)
(defvar *select-scroll-margin* 3)

(defvar *select-map* nil)
(setq *select-map* (make-sparse-keymap))
(define-key *select-map* #\i 'select-ilogicdata-infoupdate-show)
(define-key *select-map* #\a 'select-ilogicdata-autosolve-run-with-check-current-view)
(define-key *select-map* #\v 'select-ilogicdata-view-initial-matrix)
(define-key *select-map* #\TAB 'select-view-change)
(define-key *select-map* #\SPC 'select-goto-next-position)
(define-key *select-map* #\Down 'select-goto-next-position)
(define-key *select-map* #\Up 'select-goto-previous-position)
(define-key *select-map* #\C-n 'select-goto-next-position)
(define-key *select-map* #\C-p 'select-goto-previous-position)
(define-key *select-map* #\RET 'select-return)
(define-key *select-map* #\q 'select-delete)
(define-key *select-map* '(#\C-c #\q) 'select-delete)

(defun set-select-buffer ()
  (set-buffer (get-buffer-create *buffer-select*))
  (select-mode))

(defun select-mode-line ()
  (let (relative-current-data-directory
	current-data-directory-name-list
	data-directory-name-list)
    (setq current-data-directory-name-list (split-string current-data-directory #\/))
    (setq data-directory-name-list (split-string *data-directory* #\/))
    (while data-directory-name-list
      (if (string-equal (car data-directory-name-list)
			(car current-data-directory-name-list))
	  (progn
	    (pop current-data-directory-name-list)
	    (pop data-directory-name-list))
	(return)))
    (if data-directory-name-list
	(setq relative-current-data-directory current-data-directory)
      (setq relative-current-data-directory
	    (concat "<DATA TOP>/"
		    (join-string-list "/" current-data-directory-name-list))))
    (setq title-bar-format (format nil "~A - %b: ~A"
				   *version-name* relative-current-data-directory))
    (refresh-title-bar)
    (update-mode-line)
    ))

(defun select-mode-line-update ()
  (save-excursion
    (set-buffer *buffer-select*)
    (select-mode-line)))

(defun select ()
  (interactive)
  (set-select-buffer)
  (select-create))

(defun select-mode ()
  (setq buffer-mode 'ilogic-select)
  (setq mode-name *mode-name*)
  (make-local-variable 'current-data-directory)
  (setq current-data-directory *data-directory*)
  (make-local-variable 'select-current-view)
  (setq select-current-view 'infoupdate)
  (make-local-variable 'title-bar-format)
  (select-mode-line)
  (make-local-variable '*scroll-margin*)
  (setq *scroll-margin* *select-scroll-margin*)
  (setq buffer-read-only t)
  (setq need-not-save t)
  (setq kept-undo-information nil)
  (setq auto-save nil)
  (toggle-ime nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-line-number* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-newline* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-eof* nil)
  (set-local-window-flags (selected-buffer)
			  *window-flag-cursor-line* t)
  (set-buffer-fold-width nil)
  (use-keymap *select-map*)
  (run-hooks '*select-mode-hook*))

(defun select-create ()
  (let ((buffer-read-only nil)
	class-name-list
	file-name-list
	class-directory file
	ilogicdata)
    (declare (special buffer-read-only))
    (erase-buffer (selected-buffer))

    (dolist (file-name (directory current-data-directory))
      (if (eq #\/ (schar file-name (1- (length file-name))))
	  (push (substring file-name 0 (1- (length file-name)))
		class-name-list)
	(push file-name file-name-list)))
    (setq class-name-list (stable-sort class-name-list #'string-lessp))
    (setq file-name-list (stable-sort file-name-list #'string-lessp))

    (set-text-attribute (point)
			(progn
			  (insert (format nil "[j[ɖ߂]~%"))
			  (point))
			'menu)
    (insert "-" 78)
    (insert "\n")
    (unless (string-equal current-data-directory
			  *data-directory*)
      (push ".." class-name-list))
    (dolist (class-name class-name-list)
      (setq class-directory (merge-pathnames class-name current-data-directory))
      (set-text-attribute (point)
			  (progn
			    (insert (format nil "[~A]~%" class-name))
			    (point))
			  (cons 'class class-directory)))
    (insert "-" 78)
    (insert "\n")
    (dolist (file-name file-name-list)
      (setq file (merge-pathnames file-name current-data-directory))
      (with-open-file (in file
			  :direction :input)
	(setq ilogicdata (read in nil))
	(set-text-attribute (point)
			    (progn
			      (insert (format nil "~A~%" (ilogicdata-title ilogicdata)))
			      (point))
			    (cons 'file file)
			    )
	)
      )
    (backward-char)
    (delete-char)
    (goto-char (point-min))
    (set-buffer-modified-p nil)
    ))
    

(defun select-delete ()
  (interactive)
  (delete-buffer *buffer-select*)
  (if (find-buffer *buffer-infoupdate*)
      (infoupdate-delete))
  (if (find-buffer *buffer-autosolve*)
      (autosolve-delete))
  (menu-show))

(defun select-ilogicdata-show-current-view ()
  (interactive)
  (save-excursion
    (let ((ilogicdata (select-get-current-line-ilogicdata)))
      (if ilogicdata
	  (progn
	    (case select-current-view
	      ('infoupdate
	       (select-ilogicdata-infoupdate-show))
	      ('initial-matrix
	       (select-ilogicdata-view-initial-matrix))
	      ((nil)
	       (delete-other-windows)))
	    t)
	nil))))

(defun select-goto-next-position ()
  (interactive)
  (multiple-value-bind (from to tag)
      (find-text-attribute nil :start (1+ (point)) :test #'(lambda (a b) t))
    (if from
	(progn
	  (goto-char from)
	  (select-ilogicdata-show-current-view)
	  t)
      (progn
	(multiple-value-bind (from to tag)
	    (find-text-attribute nil :start (point-min) :test #'(lambda (a b) t))
	  (if from
	      (progn
		(goto-char from)
		(select-ilogicdata-show-current-view)
		t)
	    nil))))))

(defun select-goto-previous-position ()
  (interactive)
  (let (opoint)
    (goto-bol)
    (setq opoint (point))
    (while (and (backward-line)
		(not (find-text-attribute nil :start (point) :end (1+ (point))
					  :test #'(lambda (a b) t)))))
    (multiple-value-bind (from to tag)
	(find-text-attribute nil :start (point) :end (1+ (point))
			     :test #'(lambda (a b) t))
      (if (and from (/= from opoint))
	  (progn
	    (goto-char from)
	    (select-ilogicdata-show-current-view)
	    t)
	(progn
	  (goto-char (point-max))
	  (goto-bol)
	  (while (and (not (find-text-attribute nil :start (point) :end (1+ (point))
						:test #'(lambda (a b) t)))
		      (/= (current-line-number) 1))
	    (backward-line))
	  (multiple-value-bind (from to tag)
	      (find-text-attribute nil :start (point) :end (1+ (point))
				   :test #'(lambda (a b) t))
	    (if from
		(progn
		  (goto-char from)
		  (select-ilogicdata-show-current-view)
		  t)
	      (progn
		(goto-char opoint)
		nil))))))))

(defun select-goto-next-ilogicdata ()
  (interactive)
  (let ((opoint (point)))
    (if (next-line)
	(goto-bol)
      (goto-char (point-min)))
    (multiple-value-bind (from to tag)
	(find-text-attribute 'file :start (point)
			     :test #'(lambda (symbol tag)
				       (if (consp tag) (eq symbol (car tag)) nil)))
      (if from
	  (progn
	    (goto-char from)
	    (select-ilogicdata-show-current-view)
	    t)
	(progn (goto-char opoint) nil))
      )))

(defun select-get-current-line-ilogicdata ()
  (save-excursion
    (let (ilogicdata-file
	  ilogicdata)
      (goto-bol)
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'file :start (point) :end (progn (goto-eol) (point))
			       :test #'(lambda (symbol tag)
					 (if (consp tag) (eq symbol (car tag)) nil)))
	(if from
	    (progn
	      (setq ilogicdata-file (cdr tag))
	      (with-open-file (in ilogicdata-file
				  :direction :input)
		(setq ilogicdata (read in nil)))
	      ilogicdata)
	  nil)))))

(defun select-ilogicdata-infoupdate-show ()
  (interactive)
  (save-excursion
    (let ((ilogicdata (select-get-current-line-ilogicdata)))
      (if ilogicdata
	  (progn
	    (setq select-current-view 'infoupdate)
	    (unless (= (count-windows) 2)
	      (delete-other-windows)
	      (split-window *select-height*))
	    (other-window)
	    (set-infoupdate-buffer)
	    (infoupdate-show ilogicdata)
	    (other-window)
	    t)
	nil))))

(defun select-ilogicdata-view-initial-matrix ()
  (interactive)
  (save-excursion
    (let ((ilogicdata (select-get-current-line-ilogicdata)))
      (if ilogicdata
	  (progn
	    (setq select-current-view 'initial-matrix)
	    (unless (= (count-windows) 2)
	      (delete-other-windows)
	      (split-window *select-height*))
	    (other-window)
	    (set-autosolve-buffer)
	    (unwind-protect
		(autosolve-view-initial-matrix ilogicdata)
	      (other-window))
	    t)
	nil))))

(defun select-ilogicdata-autosolve-run ()
  (interactive)
  (save-excursion
    (let ((ilogicdata (select-get-current-line-ilogicdata)))
      (if ilogicdata
	  (progn
	    (setq select-current-view 'initial-matrix)
	    (unless (= (count-windows) 2)
	      (delete-other-windows)
	      (split-window *select-height*))
	    (other-window)
	    (set-autosolve-buffer)
	    (unwind-protect
		(autosolve-run ilogicdata)
	      (other-window))
	    t)
	nil))))

(defun select-ilogicdata-autosolve-run-with-check-current-view ()
  (interactive)
  (save-excursion
    (let ((ilogicdata (select-get-current-line-ilogicdata)))
      (if ilogicdata
	  (progn
	    (case select-current-view
	      ('initial-matrix
	       (select-ilogicdata-autosolve-run))
	      (t
	       (setq select-current-view 'initial-matrix)
	       (select-ilogicdata-show-current-view)))
	    t)
	nil))))

(defun select-create-by-tag ()
  (let (class-directory)
    (save-excursion
      (goto-bol)
      (multiple-value-bind (from to tag)
	  (find-text-attribute 'class :start (point) :end (progn (goto-eol) (point))
			       :test #'(lambda (symbol tag)
					 (if (consp tag) (eq symbol (car tag)) nil)))
	(if from
	    (progn
	      (setq class-directory (cdr tag))
	      (delete-other-windows)
	      (setq current-data-directory class-directory)
	      (select-mode-line-update)
	      (select-create)
	      t)
	  nil)))))

(defun select-view-change ()
  (interactive)
  (save-excursion
    (let ((ilogicdata (select-get-current-line-ilogicdata)))
      (if ilogicdata
	  (progn
	    (case select-current-view
	      ('infoupdate
	       (setq select-current-view 'initial-matrix))
	      ('initial-matrix
	       (setq select-current-view nil))
	      ((nil)
	       (setq select-current-view 'infoupdate)))
	    (select-ilogicdata-show-current-view)
	    t)
	nil))))

(defun select-delete-by-tag ()
  (interactive)
  (save-excursion
    (goto-bol)
    (multiple-value-bind (from to tag)
	(find-text-attribute 'menu :start (point) :end (progn (goto-eol) (point)))
      (if from
	  (progn
	    (select-delete)
	    t)
	nil))))

(defun select-return ()
  (interactive)
  (cond
   ((select-delete-by-tag))
   ((select-ilogicdata-autosolve-run-with-check-current-view))
   ((select-create-by-tag))))

